/*  Inti: Integrated Foundation Classes
 *  Copyright (C) 2002-2003 The Inti Development Team.
 *
 *  messagedialog.cc - GtkMessageDialog C++ wrapper implementation
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "messagedialog.h"
#include "private/messagedialog_p.h"
#include <gtk/gtklabel.h>

using namespace Inti;

/*  Gtk::MessageDialog
 */

Gtk::MessageDialog::MessageDialog(GtkMessageDialog *dialog, bool reference)
: Dialog((GtkDialog*)dialog, reference)
{
}

namespace {

using namespace Gtk;

void
set_dialog_data(MessageDialog *dialog, Window *parent, DialogFlagsField flags)
{
	if (parent)
		dialog->set_transient_for(parent);
	if (flags & DIALOG_MODAL)
		dialog->set_modal(true);
	if (flags & DIALOG_DESTROY_WITH_PARENT)
		dialog->set_destroy_with_parent(true);
	if (flags & DIALOG_NO_SEPARATOR)
		dialog->set_has_separator(false);
}

} // namespace

Gtk::MessageDialog::MessageDialog(Window *parent, DialogFlagsField flags)
{
	set_dialog_data(this, parent, flags);
}

Gtk::MessageDialog::MessageDialog(MessageType type, ButtonType buttons, Window *parent, DialogFlagsField flags)
: Dialog((GtkDialog*)MessageDialogClass::create())
{
	set("message_type", (GtkMessageType)type, "buttons", (GtkButtonsType)buttons, 0);
	set_dialog_data(this, parent, flags);
}

Gtk::MessageDialog::MessageDialog(MessageType type, ButtonType buttons, const String& message, Window *parent, DialogFlagsField flags)
: Dialog((GtkDialog*)MessageDialogClass::create())
{
	set("message_type", (GtkMessageType)type, "buttons", (GtkButtonsType)buttons, 0);
	set_dialog_data(this, parent, flags);
	set_message(message);
}

Gtk::MessageDialog::~MessageDialog()
{
}

GtkMessageDialogClass*
Gtk::MessageDialog::gtk_message_dialog_class() const
{
	return get_class<GtkMessageDialogClass>();
}

Gtk::MessageDialog::operator GtkMessageDialog* () const
{
	return this ? gtk_message_dialog() : 0;
}

void
Gtk::MessageDialog::set_message(const String& message)
{
	gtk_label_set_text(GTK_LABEL(gtk_message_dialog()->label), message.c_str());
}

void
Gtk::MessageDialog::set_message(const char *message_format, ...)
{
	g_return_if_fail(message_format != 0);
	va_list args;
	va_start(args, message_format);
	char* msg = g_strdup_vprintf(message_format, args);
	va_end(args);
	gtk_label_set_text(GTK_LABEL(gtk_message_dialog()->label), msg);
	g_free(msg);
}

/*  Gtk::MessageDialogClass
 */

void
Gtk::MessageDialogClass::init(GtkMessageDialogClass *g_class)
{
	DialogClass::init((GtkDialogClass*)g_class);
}

GType
Gtk::MessageDialogClass::get_type()
{
	static GType type = 0;
	if (!type)
	{
		type = G::TypeInstance::register_type(GTK_TYPE_MESSAGE_DIALOG, (GClassInitFunc)init);
	}
	return type;
}

void*
Gtk::MessageDialogClass::create()
{
	return g_object_new(get_type(), 0);
}

/*  Properties
 */

const Gtk::MessageDialog::ButtonsPropertyType Gtk::MessageDialog::buttons_property("buttons");

const Gtk::MessageDialog::MessageTypePropertyType Gtk::MessageDialog::message_type_property("message_type");

